/**
	@author Emilian Miron, Universitatea Bucuresti
	@comment 
		acoperirea cu numar minim de drumuri ce se pot intersecta
		a varfurilor unui graf orientat
**/

#include <stdio.h>
#include <queue>

using std::queue;

#define maxn 128
#define maxv (2*maxn + 4)

struct list {
	int v, cap, flow;
	struct list *nxt, *dup;
};

typedef struct list list, *plist;

plist my_alloc () 
{ return (plist) malloc (sizeof (list)); }

int N;

#define in(t) ((t)*2)
#define out(t) ((t)*2 + 1)

#define vsrc (N*2 + 2)
#define vdst (N*2 + 3)

plist adj[maxv];

void alloc_edge (plist &nou, plist &dup, int i, int j)
{
	nou = my_alloc (), dup = my_alloc ();
	nou->v = j; dup->v = i;
	nou->dup = dup; dup->dup = nou;
	nou->nxt = adj[i]; dup->nxt = adj[j];
	adj[i] = nou; adj[j] = dup;
}

void init(void)
{
	int M, i, j;
	plist nou, dup;
	
freopen("drumuri.in", "r", stdin);
	scanf ("%d %d", &N, &M);
	for (; M--;) {	
		scanf ("%d %d", &i, &j);
		alloc_edge (nou, dup, out(i), in(j));
		nou->cap = N; nou->flow = 0; 
		dup->cap = 0; dup->flow = 0;
	}
	
	for (i = 1; i <= N; i++) {
		/* pornim de la un flux prin fiecare nod, dar 
			omitem fluxul de pe muchia back a nodului, 
			ca sa se respecte restrictia */
		alloc_edge (nou, dup, in(i), out(i));
		nou->cap = N; nou->flow = 0; 
		dup->cap = 0; dup->flow = 0;
		
		alloc_edge (nou, dup, vsrc, in(i));
		nou->cap = N; nou->flow = 1;
		dup->cap = 0; dup->flow = -1;
		
		alloc_edge (nou, dup, out(i), vdst);
		nou->cap = N; nou->flow = 1;
		dup->cap = 0; dup->flow = -1;
	}
}

/* pompam flux inapoi din destinatie spre sursa */

bool augment () 
{
	int i;
	plist nxt;
	queue <int> q;
	bool viz[maxv];
	plist edge[maxv];
	
	for (i = 1; i <= vdst; i++)
		viz[i] = false;
	
	viz[vdst] = true;
	edge[vdst] = 0;
	q.push(vdst);
	
	for (; ! q.empty(); q.pop()) {
		int cur = q.front();
		for (nxt = adj[cur]; nxt; nxt = nxt->nxt)
			if (! viz[nxt->v] && nxt->cap - nxt->flow > 0) {
				q.push(nxt->v);
				viz[nxt->v] = true;
				edge[nxt->v] = nxt;
			}
	}
	
	if (viz[vsrc] == false)
		return false;
	
	// printf ("augment:");
	for (nxt = edge[vsrc]; nxt; nxt = edge[nxt->dup->v]) {
		nxt->flow ++, nxt->dup->flow --;
		// printf (" %d(%s)", (nxt->dup->v/2), nxt->dup->v%2 ? "out" : "in");
	}
	// printf ("\n");
	
	return true;
}

void solve (void)
{
	int flow;
	
	for (flow = N; augment(); flow--);
	freopen("drumuri.out","w",stdout);
	printf ("%d\n", flow);
}

int main(void)
{
	init();
	solve();
	
	return 0;
}
